@@ -6,17 +6,22 @@ import hashlib
 import json
 from json.encoder import JSONEncoder
 import datetime
+import mimetypes
 import os
+import stat
 from django import forms
 from django.conf import settings
 from django.core.context_processors import csrf
 from django.core.exceptions import PermissionDenied
-from django.http import HttpResponse, Http404, QueryDict
+from django.db.models import Q
+from django.http import HttpResponse, Http404, QueryDict, StreamingHttpResponse, HttpResponseNotModified
 from django.shortcuts import render_to_response, get_object_or_404
 from django.template import RequestContext
 from django.utils.timezone import utc
 from django.utils.translation import ugettext_lazy as _
 from django.views.decorators.csrf import csrf_exempt
+import re
+from django.views.static import was_modified_since
 from pythonnest.models import Package, Release, ReleaseDownload, PackageRole, Classifier, Dependence, MEDIA_ROOT_LEN, \
     PackageType
 
@@ -49,28 +54,8 @@ def version_json(request, package_name, version):
 
 
 class SearchForm(forms.Form):
-    """Sample form, with three fields."""
-    # pylint: disable=R0903
-    # pylint: disable=W0232
-    search = forms.CharField(label=_('Pattern'), max_length=200, min_length=3,
-                             widget=forms.widgets.TextInput(attrs={'placeholder': _('Please enter a word'), }))
-
-
-def index(request):
-    """Index view, displaying and processing a form."""
-    results = []
-    has_results = False
-    if request.method == 'POST':
-        form = SearchForm(request.POST)
-        if form.is_valid():  # pylint: disable=E1101
-            results = Package.objects.filter(name__icontains=form.cleaned_data['search']).select_related()
-            has_results = True
-    else:
-        form = SearchForm()
-    base_url = request.build_absolute_uri()
-    template_values = {'form': form, 'results': results, 'has_results': has_results, 'base_url': base_url, }
-    template_values.update(csrf(request))  # prevents cross-domain requests
-    return render_to_response('index.html', template_values, RequestContext(request))
+    """Upload form"""
+    search = forms.CharField(max_length=255)
 
 
 def simple(request, package_name=None, version=None):
@@ -186,3 +171,86 @@ def setup(request):
         download.log()
     template_values = {}
     return render_to_response('simple.html', template_values, RequestContext(request))
+
+
+def static_serve(request, path, document_root=None):
+    if document_root is None:
+        document_root = settings.STATIC_ROOT
+    filename = os.path.abspath(os.path.join(document_root, path))
+    if not filename.startswith(document_root):
+        raise Http404
+    if settings.USE_XSENDFILE:
+        return xsendfile(request, filename)
+    return sendfile(request, filename)
+
+
+def xsendfile(request, filename):
+    response = HttpResponse()
+    response['X-Sendfile'] = filename.encode('utf-8')
+    return response
+
+
+range_re = re.compile(r'bytes=(\d+)-(\d+)')
+
+
+def sendfile(request, filename):
+    # Respect the If-Modified-Since header.
+    if not os.path.isfile(filename):
+        raise Http404
+    statobj = os.stat(filename)
+    if not was_modified_since(request.META.get('HTTP_IF_MODIFIED_SINCE'),
+                              statobj[stat.ST_MTIME], statobj[stat.ST_SIZE]):
+        return HttpResponseNotModified()
+    content_type = mimetypes.guess_type(filename)[0]
+    range_ = request.META.get('HTTP_RANGE', '')
+    t = range_re.match(range_)
+    size = os.path.getsize(filename)
+    start = 0
+    end = size - 1
+    if t:
+        start, end = int(t.group(1)), int(t.group(2))
+    if end - start + 1 < size:
+        obj = open(filename, 'rb')
+        obj.seek(start)
+        response = HttpResponse(obj.read(end - start + 1), content_type=content_type, status=206)
+        response['Content-Range'] = 'bytes %d-%d/%d' % (start, end, size)
+    else:
+        obj = open(filename, 'rb')
+        return StreamingHttpResponse(obj, content_type=content_type)
+    response['Content-Length'] = end - start + 1
+    #response["Last-Modified"] = http_date(statobj[stat.ST_MTIME])
+    return response
+
+
+def index(request):
+    """Index view, displaying and processing a form."""
+    search = SearchForm(request.GET)
+    if search.is_valid():
+        pattern = search.cleaned_data['search']
+        if len(pattern) > 2:
+            packages = Package.objects.filter(name__icontains=pattern).select_related()
+        else:
+            packages = Package.objects.filter(name__iexact=pattern).select_related()
+        template_values = {'results': packages.distinct().select_related(), 'title': _('PythonNest')}
+        return render_to_response('search_result.html', template_values, RequestContext(request))
+    base_url = request.build_absolute_uri('/')
+    template_values = {'base_url': base_url, }
+    template_values.update(csrf(request))  # prevents cross-domain requests
+    return render_to_response('index.html', template_values, RequestContext(request))
+
+
+def show_package(request, package_id, release_id=None):
+    package = get_object_or_404(Package, id=package_id)
+    roles = PackageRole.objects.filter(package=package).select_related()
+    releases = list(Release.objects.filter(package=package).order_by('-id').select_related())
+    if release_id is None:
+        release = releases[0] if releases else None
+    else:
+        release = get_object_or_404(Release, id=release_id, package=package)
+    downloads = ReleaseDownload.objects.filter(release=release).order_by('filename')
+    template_values = {'title': _('PythonNest — %(p)s') % {'p': package.name},
+                       'package': package, 'roles': roles,
+                       'is_editable': request.user in set([x.user for x in roles]),
+                       'release': release, 'releases': releases, 'downloads': downloads,
+                       }
+    return render_to_response('show_package.html', template_values, RequestContext(request))
